﻿using C1.Chart.Annotation;
using C1.Xaml.Chart;
using C1.Xaml.Chart.Annotation;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Windows.Foundation;
using Windows.UI.Input.Inking;
using Windows.UI.Input.Inking.Analysis;
using Windows.UI.Xaml.Media;

namespace FlexChartInkDemo
{
    public class InkShapeConverter
    {
        private InkPresenter inkPresenter;
        private List<IInkAnalysisNode> drawings;
        private List<IInkAnalysisNode> lines;

        public InkShapeConverter(InkPresenter presenter)
        {
            this.inkPresenter = presenter;
        }

        public async Task<List<IAnnotationBase>> ProcessInkAsync()
        {
            var analyzer = new InkAnalyzer();
            var strokes = inkPresenter.StrokeContainer.GetStrokes();
            var annotations = new List<IAnnotationBase>();

            if (strokes.Count > 0)   //Strokes to process
            {
                analyzer.AddDataForStrokes(strokes);
                var analysisResult = await analyzer.AnalyzeAsync();
                if (analysisResult.Status == InkAnalysisStatus.Updated)
                {
                    //Identify shapes 
                    drawings = analyzer.AnalysisRoot.FindNodes(InkAnalysisNodeKind.InkDrawing).ToList();
                    //Identify words & lines
                    lines = analyzer.AnalysisRoot.FindNodes(InkAnalysisNodeKind.Line).ToList();
                    var shapes = ProcessShapes();
                    annotations.AddRange(shapes);

                    var texts = ProcessText();
                    annotations.AddRange(texts);
                }
            }
            return annotations;
        }

        private IList<Shape> ProcessShapes()
        {
            var shapes = new List<Shape>();
            foreach (InkAnalysisInkDrawing drawing in drawings)
            {
                Shape shape = null;
                if (drawing.DrawingKind == InkAnalysisDrawingKind.Drawing)
                {
                    if (drawing.GetStrokeIds().Count == 1)   //Single stroke 
                    {
                        shape = this.ConvertToLineAnnotation(drawing);
                    }
                }
                else  //Any of the Recognized Shapes
                {
                    //Text which would act as the content of the Annotation
                    var line = (InkAnalysisLine)lines.FirstOrDefault(w => drawing.BoundingRect.Contains(w.BoundingRect));
                    shape = this.ConvertToShape(drawing, line);
                    lines.Remove(line);
                    RemoveProcessedNode(line);
                }
                if (shape != null)
                {
                    shapes.Add(shape);
                    RemoveProcessedNode(drawing);
                }
            }
            return shapes;
        }

        private IList<Text> ProcessText()
        {
            var strings = new List<Text>();
            foreach (InkAnalysisLine line in lines)
            {
                //Convert to Text Annotation
                strings.Add(this.ConvertToTextAnnotation(line));
                RemoveProcessedNode(line);
            }
            return strings;
        }

        private void RemoveProcessedNode(IInkAnalysisNode node)
        {
            if (node == null)
                return;
            foreach (var strokeId in node.GetStrokeIds())
            {
                var stroke = inkPresenter.StrokeContainer.GetStrokeById(strokeId);
                if(stroke!=null )
                    stroke.Selected = true;
            }
            inkPresenter.StrokeContainer.DeleteSelected();
        }


        #region Converters
        private Text ConvertToTextAnnotation(InkAnalysisLine node)
        {
            var rect = node.BoundingRect;
            var x = rect.Left;
            var y = rect.Top - rect.Height / 2;
            var text = node.RecognizedText;
            var textAnno = new Text(text)
            {
                Attachment = AnnotationAttachment.Absolute,
                Location = new Point(x, y)
            };
            textAnno.Style = this.GetStyle(node);
            return textAnno;
        }

        private Line ConvertToLineAnnotation(InkAnalysisInkDrawing drawing)
        {
            var strokeId = drawing.GetStrokeIds().First();
            var stroke = inkPresenter.StrokeContainer.GetStrokeById(strokeId);
            var inkPoints = stroke.GetInkPoints();
            var points = inkPoints.Select(x => x.Position);
            var bestfit = Utils.Utilities.GenerateLinearBestFit(points);

            var line = new Line
            {
                Start = bestfit.First(),
                End = bestfit.Last()
            };
            line.Style = this.GetStyle(drawing);
            return line;
        }

        private Shape ConvertToShape(InkAnalysisInkDrawing drawing,InkAnalysisLine line = null)
        {
            var rect = drawing.BoundingRect;
            var x = rect.Left + rect.Width / 2;
            var y = rect.Top + rect.Height / 2;
            var text = line != null ? line.RecognizedText : string.Empty;

            Shape shape;
            switch (drawing.DrawingKind)
            {
                case InkAnalysisDrawingKind.Circle:
                    var circle = new Circle(text)
                    {
                        Radius = rect.Width / 2
                    };
                    shape = circle;
                    break;
                case InkAnalysisDrawingKind.Ellipse:
                    var ellipse = new Ellipse(text)
                    {
                        Width = rect.Width,
                        Height = rect.Height
                    };
                    shape = ellipse;
                    break;
                case InkAnalysisDrawingKind.Square:
                    var square = new Square(text)
                    {
                        Length = rect.Width
                    };
                    shape = square;
                    break;
                case InkAnalysisDrawingKind.Rectangle:
                    var rectangle = new Rectangle(text)
                    {
                        Width = rect.Width,
                        Height = rect.Height
                    };
                    shape = rectangle;
                    break;
                default:
                    var poly = new Polygon(text);
                    foreach (var point in drawing.Points)
                        poly.Points.Add(point);
                    shape = poly;
                    break;
            }
            shape.Location = new Point(x, y);
            shape.Attachment = AnnotationAttachment.Absolute;
            shape.Style = this.GetStyle(drawing);
            shape.ContentStyle = this.GetStyle(line);
            return shape;
        }

        private ChartStyle GetStyle(IInkAnalysisNode node)
        {
            if (node == null)
                return null;

            var strokeId = node.GetStrokeIds().First();
            var stroke = inkPresenter.StrokeContainer.GetStrokeById(strokeId);
            if (stroke == null)
                return null;
            var color = stroke.DrawingAttributes.Color;
            var thickness = stroke.DrawingAttributes.Size.Width;
            var style = new ChartStyle();
            
            style.Stroke = new Windows.UI.Xaml.Media.SolidColorBrush(color);
            style.StrokeThickness = thickness;
            style.FontFamily = new FontFamily("Segoe Script");
            style.FontSize = node.BoundingRect.Height;
            style.Fill = new SolidColorBrush(Windows.UI.Colors.Transparent);
            return style;
        }
        #endregion
    }

    public static class Extensions
    {
        public static bool IsKnownAnnotationShape(this InkAnalysisInkDrawing drawing)
        {
            return drawing.DrawingKind == InkAnalysisDrawingKind.Circle ||
                drawing.DrawingKind == InkAnalysisDrawingKind.Ellipse ||
                drawing.DrawingKind == InkAnalysisDrawingKind.Rectangle ||
                drawing.DrawingKind == InkAnalysisDrawingKind.Square;
        }

        public static bool Contains(this Rect rect, Rect innerRect)
        {
            var topLeft = new Point(innerRect.Left, innerRect.Top);
            var bottomRight = new Point(innerRect.Right, innerRect.Bottom);
            return rect.Contains(topLeft) && rect.Contains(bottomRight);
        }
    }
}
